package com.sillycycle.bagleyd.threed;

/*
 * @(#)ThreedFrame.java
 *
 * Copyright 1994 - 2020  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.sillycycle.bagleyd.util.ArgumentParser;
import com.sillycycle.bagleyd.util.Colour;
import com.sillycycle.bagleyd.util.ColumnLayout;
import com.sillycycle.bagleyd.util.ComponentUtil;
import com.sillycycle.bagleyd.util.Icon;
import com.sillycycle.bagleyd.util.OKDialog;
import com.sillycycle.bagleyd.util.Slider;

/**
 * The <code>ThreedFrame</code> class is to be embedded in a Web page
 * or run as an application.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/threed.html
 */

public class ThreedFrame extends JFrame implements ActionListener {
	private static final long serialVersionUID = 42L;
	public static final String NL = System.getProperty("line.separator");
	static final int MAXTHREED = 8;
	static final int MIN_SPEED = 1;
	static final int MAX_SPEED = 50;
	static final int NEVER = -1;
	static final int MAX_DEGREES = (ThreedInterface.NUM_DEGREES - 1);
	static final String[] COMMAND_STRINGS = new String[] {
		"Description ?", "Features !", "References ^", "About",
		"Exit"};
	static final char[] COMMAND_CHARS = new char[] {
		'?', '!', '^', 'a', 'x'};

	static ArgumentParser argumentParser = null;
	Icon mediumIcon = null;
	Icon largeIcon = null;
	int widthValue = 512;
	int heightValue = 512;
	static int windowXValue = 0;
	static int windowYValue = 0;
	String monoString = "YGRBKN";
	Color[] shadeColor = new Color[] {
		Color.YELLOW, Color.GREEN, Color.RED,
		Color.BLUE, Color.BLACK, Colour.DARK_BROWN,
	};
	Color borderColor = Color.LIGHT_GRAY;
	Color fgColor = Color.CYAN;
	Color bgColor = Colour.STEEL_BLUE;
	boolean sound = false;
	boolean controlValue = true;
	int xValue = 0;
	int yValue = 0; int zValue = 50;
	int thetaValue = ThreedInterface.MIN_DEGREES;
	int phiValue = ThreedInterface.MIN_DEGREES;
	int psiValue = ThreedInterface.MIN_DEGREES;
	int delayValue = 10;
	int speedValue = MAX_SPEED + MIN_SPEED - 1 - delayValue;
	int objectValue = 6; /* Rhombic Triacontahedron */
	boolean surfaceValue = true;
	boolean debug = false;
	JPanel topPanel = new JPanel();
	JCheckBox surfaceCheckBox = new JCheckBox("Surface");
	Threed threed;
	/*int movesDisplay = 0;
	String recordDisplay = "NEVER";
	String messageDisplay = "Play Threed! (use mouse or keypad)";
	JTextField movesValueLabel = new JTextField(Integer.toString(
		movesDisplay));
	JTextField recordValueLabel = new JTextField(recordDisplay);
	Label messageLabel = new JLabel(messageDisplay, JLabel.CENTER);*/
	OKDialog aDialog = null;
	StringBuilder stringBuilderSave;
	JComboBox<Object> objectComboBox = new JComboBox<>();
	JMenuBar menuBar;
	JMenuItem detachMenuItem;
	JPanel mainPanel = new JPanel();
	JPanel secondaryPanel = new JPanel();
	JFrame frame;
	/*Spinner xChanger = null, yChanger = null, zChanger = null;
	Spinner thetaChanger = null, phiChanger = null, psiChanger = null;*/
	Slider xChanger = null, yChanger = null, zChanger = null;
	Slider thetaChanger = null, phiChanger = null, psiChanger = null;
	/*Slider speedChanger = null;*/

	public String getThreedString() {
		return this.stringBuilderSave.toString();
	}

	public void writeThreedString(String s) {
		threed.stringSave = s;
	}

	public String getOpt(String string) {
		return argumentParser.getOption(string);
	}

	public int getOpt(String param, int value) {
		String string = getOpt(param);

		if (string == null)
			return value;
		try {
			return Integer.parseInt(string);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}

	public boolean getOpt(String param, boolean value) {
		String string = getOpt(param);

		if (string == null)
			return value;
		try {
			if (string.equalsIgnoreCase("true") ||
					string.equalsIgnoreCase("yes") ||
					string.equalsIgnoreCase("on"))
				return true;
			else if (string.equalsIgnoreCase("false") ||
					string.equalsIgnoreCase("no") ||
					string.equalsIgnoreCase("off"))
				return false;
			return (Integer.parseInt(string) != 0);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}

	public Color getOpt(String param, Color value) {
		String string = getOpt(param);

		if (string == null)
			return value;
		try {
			return Color.decode(string);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}

	public void getOpts() {
		widthValue = getOpt("windowWidth", widthValue);
		heightValue = getOpt("windowHeight", heightValue);
		windowXValue = getOpt("windowX", windowXValue);
		windowYValue = getOpt("windowY", windowYValue);
		for (int i = 0; i < shadeColor.length; i++)
			shadeColor[i] = getOpt("shadeColor" + i,
				shadeColor[i]);
		borderColor = getOpt("borderColor", borderColor);
		fgColor = getOpt("fgColor", fgColor);
		bgColor = getOpt("bgColor", bgColor);
		controlValue = getOpt("control", controlValue);
		surfaceValue = getOpt("surface", surfaceValue);
		objectValue = getOpt("object", objectValue);
		xValue = getOpt("x", xValue);
		if (xValue < ThreedInterface.MINDISTANCE)
			xValue = ThreedInterface.MINDISTANCE;
		yValue = getOpt("y", yValue);
		if (yValue < ThreedInterface.MINDISTANCE)
			yValue = ThreedInterface.MINDISTANCE;
		zValue = getOpt("z", zValue);
		if (zValue < ThreedInterface.MINDISTANCE)
			zValue = ThreedInterface.MINDISTANCE;
		thetaValue = getOpt("theta", thetaValue);
		if (thetaValue < ThreedInterface.MIN_DEGREES)
			thetaValue = ThreedInterface.MIN_DEGREES;
		else if (thetaValue > ThreedInterface.MAX_DEGREES)
			thetaValue = ThreedInterface.MAX_DEGREES;
		phiValue = getOpt("phi", phiValue);
		if (phiValue < ThreedInterface.MIN_DEGREES)
			phiValue = ThreedInterface.MIN_DEGREES;
		else if (phiValue > ThreedInterface.MAX_DEGREES)
			phiValue = ThreedInterface.MAX_DEGREES;
		psiValue = getOpt("psi", psiValue);
		if (psiValue < ThreedInterface.MIN_DEGREES)
			psiValue = ThreedInterface.MIN_DEGREES;
		else if (psiValue > ThreedInterface.MAX_DEGREES)
			psiValue = ThreedInterface.MAX_DEGREES;
		delayValue = getOpt("delay", delayValue);
	}

	public void showMessage(String msg) {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				ThreedInterface.TITLE, msg, "OK",
				mediumIcon, false);
			aDialog.setBounds(this.getSize().width / 2,
				this.getSize().height / 2, 350, 120);
			aDialog.setVisible(true);
		}
	}

	public void descriptionHelp() {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				ThreedInterface.TITLE + ": Description",
"A simple 3D viewer (though may need some work)." + NL +
" " + NL +
"The roll part does not seem to be quite right and may not" + NL +
"track well.  Of note is the mouse to 3D surface detection." + NL +
" " + NL +
"In the sample data, Cubes and F16 are big, so the polyhedrons" + NL +
"are scaled up to them in the data file." + NL,
				"OK", largeIcon, true);
			aDialog.setBackground(Color.white);
			aDialog.setVisible(true);
		}
	}

	public void featuresHelp() {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				ThreedInterface.TITLE + ": Features",
"Press \"I\" or \"i\" keys to move shape in (\".\", \"5\" also work)." + NL +
" " + NL +
"Press \"O\" or \"o\" keys to move shape out (\"/\", \"0\" also work)." + NL +
" " + NL +
"Press \"S\" or \"s\" keys to change surface." + NL +
" " + NL +
"Press \"B\" or \"b\" keys to change object." + NL +
" " + NL +
"Press \">\" or \".\" keys to speed up the movement of the tiles." + NL +
" " + NL +
"Press \"<\" or \",\" keys to slow up the movement of the tiles." + NL +
" " + NL +
"Press \"@\" key to toggle the sound." + NL +
" " + NL +
"Use the key pad or arrow keys to move without the" + NL +
"mouse." + NL +
"Key pad (better with Num Lock on) is defined for Threed as:" + NL +
"    /       Out" + NL +
"    8       Up" + NL +
"    ^" + NL +
"4<5>6  Left, In, Right" + NL +
"    v" + NL +
"    2       Down" + NL,
				"OK", largeIcon, true);
			aDialog.setBackground(Color.white);
			aDialog.setVisible(true);
		}
	}

	public void referencesHelp() {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				ThreedInterface.TITLE + ": References",
"Original code by" + NL +
"Thomas W. Olsen \"The C Users Journal\"." + NL +
"Andre LaMothe \"Black Art of 3D Game Programming\"." + NL +
"Arash Partow \"http://www.partow.net\".",
				"OK", mediumIcon, false);
			aDialog.setVisible(true);
		}
	}

	public void aboutHelp() {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				ThreedInterface.TITLE + ": About",
"Java ThreedFrame Version 8.3.0" + NL +
"Send bugs (reports or fixes) to the author:" +
"David Bagley <bagleyd AT verizon.net>" + NL +
"The latest version is at: " +
"https://www.sillycycle.com/threed.html" + NL,
				"OK", mediumIcon, false);
			aDialog.setVisible(true);
		}
	}

	public void processKey(char key) {
		threed.processKey(key);
	}

	public Icon getIcon(String string, int sx, int sy) {
		URL url = ThreedFrame.class.getResource("/" + string);

		if (url == null)
			return null;
		return new Icon(Toolkit.getDefaultToolkit().
			getImage(url), sx, sy);
	}

	@Override
	public void setSize(int width, int height) {
		if (threed == null)
			return;
		threed.setBounds(0, 0, width, height);
		/*repaint();*/ // causes flashing
	}

	@Override
	public void paint(Graphics g) {
		setSize(secondaryPanel.getSize().width,
			secondaryPanel.getSize().height - topPanel.getSize().height);
		super.paint(g);
	}

	public void shuffleDown() {
		ComponentUtil.findJFrame(this).toBack();
	}

	public void quit() {
		System.exit(0);
	}

	class XChangeListener implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent event) {
			int oldValue = xValue;

			xValue = xChanger.getValue();
			if (oldValue != xValue) {
				threed.setSizeX(xValue);
				/*movesDisplay = 0;
				movesValueLabel.setText(Integer.toString(
					movesDisplay));*/
			}
		}
	}

	class YChangeListener implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent event) {
			int oldValue = yValue;

			yValue = yChanger.getValue();
			if (oldValue != yValue) {
				threed.setSizeY(yValue);
				/*movesDisplay = 0;
				movesValueLabel.setText(Integer.toString(
					movesDisplay));*/
			}
		}
	}

	class ZChangeListener implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent event) {
			int oldValue = zValue;

			zValue = zChanger.getValue();
			if (oldValue != zValue) {
				threed.setSizeZ(zValue);
				/*movesDisplay = 0;
				movesValueLabel.setText(Integer.toString(
					movesDisplay));*/
			}
		}
	}

	class ThetaChangeListener implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent event) {
			int oldValue = thetaValue;

			thetaValue = thetaChanger.getValue();
			if (oldValue != thetaValue) {
				threed.setSizeTheta(thetaValue);
				/*movesDisplay = 0;
				movesValueLabel.setText(Integer.toString(
					movesDisplay));*/
			}
		}
	}

	class PhiChangeListener implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent event) {
			int oldValue = phiValue;

			phiValue = phiChanger.getValue();
			if (oldValue != phiValue) {
				threed.setSizePhi(phiValue);
				/*movesDisplay = 0;
				movesValueLabel.setText(Integer.toString(
					movesDisplay));*/
			}
		}
	}

	class PsiChangeListener implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent event) {
			int oldValue = psiValue;

			psiValue = psiChanger.getValue();
			if (oldValue != psiValue) {
				threed.setSizePsi(psiValue);
				/*movesDisplay = 0;
				movesValueLabel.setText(Integer.toString(
					movesDisplay));*/
			}
		}
	}

	/*class SpeedChangeListener implements ChangeListener {
		public void stateChanged(ChangeEvent event) {
			int oldValue = speedValue;

			speedValue = speedChanger.getValue();
			if (oldValue != speedValue) {
				threed.setDelay(MAX_SPEED + MIN_SPEED - 1 -
					speedValue);
			}
		}
	}*/

	class ComboBoxActionListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent event) {
			int objectIndex = objectComboBox.getSelectedIndex();

			if (objectIndex != -1 && objectValue != objectIndex) {
				objectValue = objectIndex;
				threed.setobjectThreeD(objectValue);
			}
		}
	}

	class SurfaceItemListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent event) {
			boolean oldValue = surfaceValue;

			surfaceValue = surfaceCheckBox.isSelected();
			if (oldValue != surfaceValue) {
				threed.setsurfaceThreeD(surfaceValue);
			}
		}
	}

	public void setXChanger(int value) {
		if (xValue != value) {
			xValue = value;
			if (xChanger != null)
				xChanger.setValue(xValue);
		}
	}

	public void setYChanger(int value) {
		if (yValue != value) {
			yValue = value;
			if (yChanger != null)
				yChanger.setValue(yValue);
		}
	}

	public void setZChanger(int value) {
		if (zValue != value) {
			zValue = value;
			if (zChanger != null)
				zChanger.setValue(zValue);
		}
	}

	public void setThetaChanger(int value) {
		if (thetaValue != value) {
			thetaValue = value;
			if (thetaChanger != null)
				thetaChanger.setValue(thetaValue);
		}
	}

	public void setPhiChanger(int value) {
		if (phiValue != value) {
			phiValue = value;
			if (phiChanger != null)
				phiChanger.setValue(phiValue);
		}
	}

	public void setPsiChanger(int value) {
		if (psiValue != value) {
			psiValue = value;
			if (psiChanger != null)
				psiChanger.setValue(psiValue);
		}
	}

	@Override
	public void actionPerformed(ActionEvent event) {
		String command = event.getActionCommand();

		for (int opt = 0; opt < COMMAND_STRINGS.length; opt++) {
			if (command.equals(COMMAND_STRINGS[opt])) {
				processKey(COMMAND_CHARS[opt]);
				return;
			}
		}
	}

	public void callbackThreeD(int reason) {
		/*messageDisplay = "                                                                                ";*/
		switch(reason) {
		case ThreedInterface.ACTION_SURFACE:
			surfaceValue = !surfaceValue;
			threed.setsurfaceThreeD(surfaceValue);
			surfaceCheckBox.setSelected(surfaceValue);
			break;
		case ThreedInterface.ACTION_OBJECT:
			objectValue = (objectValue + 1) %
				threed.getNumObjects();
			threed.setobjectThreeD(objectValue);
			objectComboBox.setSelectedItem(
				ThreedInterface.objectStrings[objectValue]);
			break;
		default:
		/*case ThreedInterface.ACTION_SPEED:
			{
				int oldDelay, delay;

				delay = threed.getDelay();
				oldDelay = delay;
				if (delay > MAX_SPEED - MIN_SPEED)
					delay = MAX_SPEED - MIN_SPEED;
				if (delay < 0)
					delay = 0;
				if (delay != oldDelay) {
					threed.setDelay(delay);
				}
				if (speedChanger != null)
					speedChanger.setValue(MAX_SPEED +
						MIN_SPEED - 1 - delay);
			}
			return; */
		}
	}

	public ThreedFrame(String title) {
		/*JPanel info1Panel = new JPanel();
		JPanel leftPanel = new JPanel();
		JPanel rightPanel = new JPanel();
		JPanel info2Panel = new JPanel();
		JPanel movesPanel = new JPanel();
		JPanel recordPanel = new JPanel();*/
		JPanel startPanel = new JPanel();
		JPanel distancePanel = new JPanel();
		JPanel anglePanel = new JPanel();
		JPanel listPanel = new JPanel();
		JPanel comboBoxPanel = new JPanel();
		JPanel canvasPanel = new JPanel();
		int objectIndex;

		try {
			mediumIcon = getIcon(ThreedInterface.ICONS[ThreedInterface.Icons.X32.ordinal()], 32, 32);
			largeIcon = getIcon(ThreedInterface.ICONS[ThreedInterface.Icons.X48.ordinal()], 48, 48);
		} catch (Exception e) {
			largeIcon = null;
		}
		getOpts();
		speedValue = MAX_SPEED + MIN_SPEED - 1 - delayValue;
		mainPanel.setLayout(new BorderLayout());
		mainPanel.setBackground(Color.white);
		if (controlValue) {
			menuBar = new JMenuBar();
			JMenu menu;
			JMenuItem menuItem;

			JPopupMenu.setDefaultLightWeightPopupEnabled(false);
			menu = new JMenu("File");
			menuItem = new JMenuItem("Exit");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_X);
			menuItem.addActionListener(this);
			menuBar.add(menu);
			menu = new JMenu("Help");
			menuItem = new JMenuItem("Description ?");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Features !");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("References ^");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("About");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_A);
			menuItem.addActionListener(this);
			menuBar.add(menu);
			//setJMenuBar(menuBar); // needs to be in a panel
			mainPanel.add(menuBar, BorderLayout.PAGE_START);
			/*leftPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
			leftPanel.add(new JLabel("Move block"));
			rightPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
			rightPanel.add(new JLabel("Randomize"));
			leftPanel.add(getIcon("mouse-l.png", 16, 16));
			rightPanel.add(getIcon("mouse-r.png", 16, 16));
			info1Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 15, 0));
			info1Panel.add(leftPanel);
			info1Panel.add(rightPanel);
			movesPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
			movesPanel.add(new JLabel("Moves"));
			movesValueLabel.setColumns(5);
			movesValueLabel.setEditable(false);
			movesPanel.add(movesValueLabel);
			recordPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
			recordPanel.add(new JLabel("Record"));
			recordValueLabel.setColumns(5);
			recordValueLabel.setEditable(false);
			recordPanel.add(recordValueLabel);
			info2Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 15, 0));
			info2Panel.add(movesPanel);
			info2Panel.add(recordPanel);*/
			distancePanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
			/*xChanger = new Spinner("X",
				xValue, ThreedInterface.MINDISTANCE,
				((xValue > ThreedInterface.MAX_DISTANCE) ? xValue :
				ThreedInterface.MAX_DISTANCE));
			yChanger = new Spinner("Y",
				yValue, ThreedInterface.MINDISTANCE,
				((yValue > ThreedInterface.MAX_DISTANCE) ? yValue :
				ThreedInterface.MAX_DISTANCE));
			zChanger = new Spinner("Z",
				zValue, ThreedInterface.MIN_DEPTH,
				((zValue > ThreedInterface.MAX_DISTANCE) ? zValue :
				ThreedInterface.MAX_DISTANCE));*/
			xChanger = new Slider("X",
				xValue, ThreedInterface.MINDISTANCE,
				((xValue > ThreedInterface.MAX_DISTANCE) ? xValue :
				ThreedInterface.MAX_DISTANCE));
			yChanger = new Slider("Y",
				yValue, ThreedInterface.MINDISTANCE,
				((yValue > ThreedInterface.MAX_DISTANCE) ? yValue :
				ThreedInterface.MAX_DISTANCE));
			zChanger = new Slider("Z",
				zValue, ThreedInterface.MIN_DEPTH,
				((zValue > ThreedInterface.MAX_DISTANCE) ? zValue :
				ThreedInterface.MAX_DISTANCE));
			distancePanel.add(xChanger);
			distancePanel.add(yChanger);
			distancePanel.add(zChanger);
			/*thetaChanger = new Spinner("Theta",
				thetaValue, ThreedInterface.MIN_DEGREES, MAX_DEGREES);
			phiChanger = new Spinner("Phi",
				phiValue, ThreedInterface.MIN_DEGREES, MAX_DEGREES);
			psiChanger = new Spinner("Psi",
				psiValue, ThreedInterface.MIN_DEGREES, MAX_DEGREES);*/
			thetaChanger = new Slider("Theta",
				thetaValue, ThreedInterface.MIN_DEGREES, MAX_DEGREES);
			phiChanger = new Slider("Phi",
				phiValue, ThreedInterface.MIN_DEGREES, MAX_DEGREES);
			psiChanger = new Slider("Psi",
				psiValue, ThreedInterface.MIN_DEGREES, MAX_DEGREES);
			anglePanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
			anglePanel.add(thetaChanger);
			anglePanel.add(phiChanger);
			anglePanel.add(psiChanger);
			/*speedChanger = new Spinner("Animation Speed",
				speedValue, MIN_SPEED, MAX_SPEED); */
			listPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
			for (objectIndex = ThreedInterface.TETRAHEDRON;
					objectIndex < ThreedInterface.objectStrings.length;
					objectIndex++) {
				objectComboBox.addItem(
					ThreedInterface.objectName(ThreedInterface.objectStrings[objectIndex]));
			}
			objectComboBox.setSelectedItem(
				ThreedInterface.objectName(ThreedInterface.objectStrings[objectValue]));
			comboBoxPanel.add(new JLabel("Object list:"));
			comboBoxPanel.add(objectComboBox);
			listPanel.add(comboBoxPanel);
			listPanel.add(surfaceCheckBox);
			surfaceCheckBox.setSelected(surfaceValue);
			topPanel.setLayout(new ColumnLayout(0, 0, 0,
				ColumnLayout.CENTER));
			/*topPanel.add(info1Panel);*/
			/*topPanel.add(info2Panel);*/
			topPanel.add(distancePanel);
			topPanel.add(anglePanel);
			topPanel.add(listPanel);
			/*topPanel.add(speedSlider);*/
			/*topPanel.add(messageLabel);*/
			startPanel.setLayout(new ColumnLayout(0, 0, 0,
				ColumnLayout.LEFT));
			startPanel.add(topPanel);
		}
		startPanel.setLayout(new ColumnLayout(0, 0, 0,
			ColumnLayout.LEFT));
		startPanel.add(topPanel);
		secondaryPanel.setLayout(new BorderLayout());
		secondaryPanel.add(startPanel, BorderLayout.PAGE_START);
		threed = new Threed(this,
			objectValue, surfaceValue,
			xValue, yValue, zValue,
			thetaValue, phiValue, psiValue,
			false, false, monoString,
			shadeColor[0], shadeColor[1], shadeColor[2],
			shadeColor[3], shadeColor[4], shadeColor[5],
			borderColor,
			fgColor, bgColor,
			MAX_SPEED + MIN_SPEED - 1 - speedValue);
		canvasPanel.setLayout(new ColumnLayout(0, 0, 0,
			ColumnLayout.LEFT));
		canvasPanel.add(threed);
		secondaryPanel.add(canvasPanel, BorderLayout.CENTER);
		mainPanel.add(secondaryPanel, BorderLayout.CENTER);
		getContentPane().add(mainPanel, BorderLayout.CENTER);
		if (controlValue) {
			xChanger.addChangeListener(new XChangeListener());
			yChanger.addChangeListener(new YChangeListener());
			zChanger.addChangeListener(new ZChangeListener());
			thetaChanger.addChangeListener(new ThetaChangeListener());
			phiChanger.addChangeListener(new PhiChangeListener());
			psiChanger.addChangeListener(new PsiChangeListener());
			/*speedSpinner.addChangeListener(new SpeedChangeListener());*/
			objectComboBox.addActionListener(new ComboBoxActionListener());
			surfaceCheckBox.addItemListener(new SurfaceItemListener());
		}
		canvasPanel.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseEntered(MouseEvent event) {
				threed.requestFocus();
			}
			@Override
			public void mouseExited(MouseEvent event) {
				threed.requestFocus();
			}
			@Override
			public void mousePressed(MouseEvent event) {
				threed.requestFocus();
			}
		});
		setTitle(title);
		/* threed.setSize(getSize().width,
			getSize().height - topPanel.getSize().height); */
		threed.setSize(widthValue, heightValue);
	}

	public static void main(String[] args) {
		String laf = UIManager.getCrossPlatformLookAndFeelClassName();

		try {
			UIManager.setLookAndFeel(laf);
		} catch (ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException e) {
			e.printStackTrace();
			System.exit(1);
		}
		List<Image> iconList = new ArrayList<>();
		for (String icon : ThreedInterface.ICONS) {
			URL url = ThreedFrame.class.getResource("/" + icon);
			if (url != null)
				iconList.add(Toolkit.getDefaultToolkit().getImage(url));
		}

		argumentParser = new ArgumentParser(args);
		if (argumentParser.hasOption("?")) {
			System.out.println("usage:" + NL +
"\t[-windowWidth={int}] [-windowHeight={int}]" + NL +
"\t[-windowX={int}] [-windowY={int}]" + NL +
"\t[-shadeColor{0|1|2|3|4|5}={int}] [-borderColor={int}]" + NL +
"\t[-fgColor={int}] [-bgColor={int}]" + NL +
"\t[-control={bool}] [-surface={bool}] [-object={int}]" + NL +
"\t[-x={int}] [-y={int}] [-z={int}]" + NL +
"\t[-theta={int}] [-phi={int}] [-psi={int}]" + NL +
"\t[-delay={int}]");
			return;
		}
		ThreedFrame frame = new ThreedFrame(ThreedInterface.TITLE);
		frame.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		if (!iconList.isEmpty()) {
			frame.setIconImages(iconList);
		}
		frame.pack();
		frame.threed.requestFocusInWindow();
		frame.setLocation(ThreedFrame.windowXValue, ThreedFrame.windowYValue);
		frame.setVisible(true);
	}
}
